﻿using System;
using System.Linq.Expressions;
using System.Threading;

namespace Nito.AsyncEx.Internal.PlatformEnlightenment
{
    partial class DefaultEnlightenmentProvider
    {
        /// <summary>
        /// The default synchronization context enlightenment, which will use <c>SynchronizationContext.SetSynchronizationContext</c> if possible.
        /// </summary>
        public sealed class SynchronizationContextEnlightenment : ISynchronizationContextEnlightenment
        {
            /// <summary>
            /// A delegate that will call <c>SynchronizationContext.SetSynchronizationContext</c>.
            /// </summary>
            private static readonly Action<SynchronizationContext> SetSynchronizationContext;

            /// <summary>
            /// Examines the current runtime and initializes the static delegates appropriately.
            /// </summary>
            static SynchronizationContextEnlightenment()
            {
                var synchronizationContextType = typeof(SynchronizationContext);
                try
                {
                    var parameter = Expression.Parameter(synchronizationContextType, "context");
                    var call = Expression.Call(synchronizationContextType, "SetSynchronizationContext", null, parameter);
                    SetSynchronizationContext = Expression.Lambda<Action<SynchronizationContext>>(call, parameter).Compile();
                }
                catch (InvalidOperationException)
                {
                    // vexing exception (Expression.Call)
                }
                catch (ArgumentException)
                {
                    // vexing exception (Expression.Lambda)
                }
            }

            void ISynchronizationContextEnlightenment.SetCurrent(SynchronizationContext context)
            {
                // According to MSDN, SetSynchronizationContext is implemented on WP70, SL3, NET40, and Win8, so we should never see this exception.
                if (SetSynchronizationContext == null)
                    throw new NotImplementedException("SynchronizationContext.SetSynchronizationContext is not implemented.");

                // However, partial-trust Silverlight may throw a MethodAccessException here.
                SetSynchronizationContext(context);
            }
        }
    }
}
