﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading.Tasks;
using Nito.AsyncEx;
using System.Linq;
using System.Threading;
using System.Threading.Tasks.Dataflow;
using System.Diagnostics.CodeAnalysis;

namespace Tests
{
    [ExcludeFromCodeCoverage]
    [TestClass]
    public class DataflowProgressUnitTests_NET40_NE
    {
        [TestMethod]
        public void ProgressReports_AreSentToBlock()
        {
            AsyncContext.Run(async () =>
            {
                var block = new BufferBlock<int>();
                IProgress<int> progress = new DataflowProgress<int>(block);
                await TaskEx.Run(() =>
                {
                    progress.Report(13);
                    progress.Report(17);
                });
                var report1 = block.Receive();
                var report2 = block.Receive();
                Assert.AreEqual(13, report1);
                Assert.AreEqual(17, report2);
            });
        }

        [TestMethod]
        public void ObserveTask_Completed_CompletesBlock()
        {
            AsyncContext.Run(async () =>
            {
                var block = new BufferBlock<int>();
                var dataflowProgress = new DataflowProgress<int>(block);
                IProgress<int> progress = dataflowProgress;
                var task = TaskEx.Run(() =>
                {
                    progress.Report(13);
                });
                dataflowProgress.ObserveTaskForCompletion(task);
                var report1 = block.Receive();
                Assert.AreEqual(13, report1);
                await block.Completion;
            });
        }

        [TestMethod]
        public void ObserveTask_Faulted_FaultsBlock()
        {
            AsyncContext.Run(async () =>
            {
                var block = new BufferBlock<int>();
                var dataflowProgress = new DataflowProgress<int>(block);
                IProgress<int> progress = dataflowProgress;
                var task = TaskEx.Run(() =>
                {
                    throw new NotImplementedException();
                });
                dataflowProgress.ObserveTaskForCompletion(task);
                await AssertEx.ThrowsExceptionAsync<NotImplementedException>(block.Completion, allowDerivedTypes: false);
            });
        }
    }
}
