﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading.Tasks;
using Nito.AsyncEx;
using System.Linq;
using System.Threading;
using System.Threading.Tasks.Dataflow;
using System.Diagnostics.CodeAnalysis;

namespace Tests
{
    [ExcludeFromCodeCoverage]
    [TestClass]
    public class DataflowProgressUnitTests_NET45_NE
    {
        [TestMethod]
        public async Task ProgressReports_AreSentToBlock()
        {
            var block = new BufferBlock<int>();
            IProgress<int> progress = new DataflowProgress<int>(block);
            await Task.Run(() =>
            {
                progress.Report(13);
                progress.Report(17);
            });
            var report1 = block.Receive();
            var report2 = block.Receive();
            Assert.AreEqual(13, report1);
            Assert.AreEqual(17, report2);
        }

        [TestMethod]
        public async Task ObserveTask_Completed_CompletesBlock()
        {
            var block = new BufferBlock<int>();
            var dataflowProgress = new DataflowProgress<int>(block);
            IProgress<int> progress = dataflowProgress;
            var task = Task.Run(() =>
            {
                progress.Report(13);
            });
            dataflowProgress.ObserveTaskForCompletion(task);
            var report1 = block.Receive();
            Assert.AreEqual(13, report1);
            await block.Completion;
        }

        [TestMethod]
        public async Task ObserveTask_Faulted_FaultsBlock()
        {
            var block = new BufferBlock<int>();
            var dataflowProgress = new DataflowProgress<int>(block);
            IProgress<int> progress = dataflowProgress;
            var task = Task.Run(() =>
            {
                throw new NotImplementedException();
            });
            dataflowProgress.ObserveTaskForCompletion(task);
            await AssertEx.ThrowsExceptionAsync<NotImplementedException>(block.Completion, allowDerivedTypes: false);
        }
    }
}
