﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading.Tasks;
using Nito.AsyncEx;
using System.Linq;
using System.Threading;
using System.Diagnostics.CodeAnalysis;

namespace Tests
{
    [ExcludeFromCodeCoverage]
    [TestClass]
    public class DeferralManagerUnitTests_NET45_NE
    {
        [TestMethod]
        public void NoDeferrals_IsCompleted()
        {
            var dm = new DeferralManager();
            var task = dm.SignalAndWaitAsync();
            Assert.IsTrue(task.IsCompleted);
        }

        [TestMethod]
        public async Task IncompleteDeferral_PreventsCompletion()
        {
            var dm = new DeferralManager();
            var deferral = dm.GetDeferral();
            await AssertEx.NeverCompletesAsync(dm.SignalAndWaitAsync());
        }

        [TestMethod]
        public async Task DeferralCompleted_Completes()
        {
            var dm = new DeferralManager();
            var deferral = dm.GetDeferral();
            var task = dm.SignalAndWaitAsync();
            Assert.IsFalse(task.IsCompleted);
            deferral.Dispose();
            await task;
        }

        [TestMethod]
        public async Task MultipleDeferralsWithOneIncomplete_PreventsCompletion()
        {
            var dm = new DeferralManager();
            var deferral1 = dm.GetDeferral();
            var deferral2 = dm.GetDeferral();
            var task = dm.SignalAndWaitAsync();
            deferral1.Dispose();
            await AssertEx.NeverCompletesAsync(task);
        }

        [TestMethod]
        public async Task TwoDeferralsWithOneCompletedTwice_PreventsCompletion()
        {
            var dm = new DeferralManager();
            var deferral1 = dm.GetDeferral();
            var deferral2 = dm.GetDeferral();
            var task = dm.SignalAndWaitAsync();
            deferral1.Dispose();
            deferral1.Dispose();
            await AssertEx.NeverCompletesAsync(task);
        }

        [TestMethod]
        public async Task MultipleDeferralsWithAllCompleted_Completes()
        {
            var dm = new DeferralManager();
            var deferral1 = dm.GetDeferral();
            var deferral2 = dm.GetDeferral();
            var task = dm.SignalAndWaitAsync();
            deferral1.Dispose();
            deferral2.Dispose();
            await task;
        }
    }
}
