﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading.Tasks;
using Nito.AsyncEx;
using System.Linq;
using System.Threading;
using System.Diagnostics.CodeAnalysis;

namespace Tests
{
    [ExcludeFromCodeCoverage]
    [TestClass]
    public class TaskFactoryExtensionsUnitTests_NET45_NE
    {
        [TestMethod]
        public async Task Run_SchedulesAction()
        {
            var factory = Task.Factory;
            TaskScheduler result = null;
            var task = factory.Run(() => { result = TaskScheduler.Current; });
            await task;
            Assert.AreSame(TaskScheduler.Default, result);
            Assert.IsTrue((task.CreationOptions & TaskCreationOptions.DenyChildAttach) == TaskCreationOptions.DenyChildAttach);
        }

        [TestMethod]
        public async Task Run_SchedulesFunc()
        {
            var factory = Task.Factory;
            TaskScheduler result = null;
            var task = factory.Run(() => { result = TaskScheduler.Current; return 13; });
            await task;
            Assert.AreSame(TaskScheduler.Default, result);
            Assert.IsTrue((task.CreationOptions & TaskCreationOptions.DenyChildAttach) == TaskCreationOptions.DenyChildAttach);
        }

        [TestMethod]
        public async Task Run_SchedulesAsyncAction()
        {
            var factory = Task.Factory;
            TaskScheduler result = null;
            var task = factory.Run(async () => { await Task.Delay(100); result = TaskScheduler.Current; });
            await task;
            Assert.AreSame(TaskScheduler.Default, result);
        }

        [TestMethod]
        public async Task Run_SchedulesAsyncFunc()
        {
            var factory = Task.Factory;
            TaskScheduler result = null;
            var task = factory.Run(async () => { await Task.Delay(100); result = TaskScheduler.Current; return 13; });
            await task;
            Assert.AreSame(TaskScheduler.Default, result);
        }
    }
}
