﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading.Tasks;
using Nito.AsyncEx;
using System.Linq;
using System.Threading;
using System.Diagnostics.CodeAnalysis;

namespace Tests
{
    [ExcludeFromCodeCoverage]
    [TestClass]
    public class CancellationTokenExtensionsUnitTests_NET45
    {
        [TestMethod]
        public void AsTask_TokenThatCannotCancel_NeverCancels()
        {
            var task = CancellationToken.None.AsTask();

            // This is testing an implementation detail.
            Assert.AreSame(TaskConstants.Never, task);
        }

        [TestMethod]
        public async Task AsTask_ConstructedCanceledToken_CancelsTask()
        {
            var token = new CancellationToken(true);
            var task = token.AsTask();

            await AssertEx.ThrowsExceptionAsync<TaskCanceledException>(task);
        }

        [TestMethod]
        public async Task AsTask_CanceledToken_CancelsTask()
        {
            var cts = new CancellationTokenSource();
            cts.Cancel();
            var task = cts.Token.AsTask();

            await AssertEx.ThrowsExceptionAsync<TaskCanceledException>(task);
        }

        [TestMethod]
        public async Task AsTask_TokenCanceled_CancelsTask()
        {
            var cts = new CancellationTokenSource();
            var task = cts.Token.AsTask();
            Assert.IsFalse(task.IsCanceled);

            cts.Cancel();
            await AssertEx.ThrowsExceptionAsync<TaskCanceledException>(task);
        }
    }
}
