﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;

[ExcludeFromCodeCoverage]
internal static class EtwExtensions
{
    public static bool DataMatches(this DebugEtw.Event @this, object filter)
    {
        var eventParams = @this.DataNames;
        foreach (var member in filter.GetType().GetProperties())
        {
            var index = eventParams.IndexOf(member.Name);
            if (index == -1)
                return false;
            if (!@this.DataValues[index].Equals(member.GetValue(filter)))
                return false;
        }
        return true;
    }

    private static readonly string eventCollectionNextIndexKey = "NextIndex_" + Guid.NewGuid().ToString("N");
    private static int GetNextIndex(DebugEtw.EventCollection collection)
    {
        var dictionary = collection.Bag as IDictionary<string, object>;
        if (dictionary.ContainsKey(eventCollectionNextIndexKey))
            return (int)dictionary[eventCollectionNextIndexKey];
        return 0;
    }
    private static void SetNextIndex(DebugEtw.EventCollection collection, int nextIndex)
    {
        var dictionary = collection.Bag as IDictionary<string, object>;
        dictionary[eventCollectionNextIndexKey] = nextIndex;
    }

    public static DebugEtw.EventCollection Expect(this DebugEtw.EventCollection @this, ExpectedEtwEvent expected, object parameters = null)
    {
#if NO
        var index = GetNextIndex(@this);
        if (index == @this.Count)
            Assert.Fail("ETW event '" + expected.SourceName + "," + expected.Id + "," + expected.Version + "' was not raised.");
        var e = @this[index];
        SetNextIndex(@this, index + 1);
        if (expected.SourceName != e.SourceName || expected.Id != e.Id || expected.Version != e.Version)
            Assert.Fail("ETW event mismatch; expected '" + expected.SourceName + "," + expected.Id + "," + expected.Version + "' but observed '" + e.SourceName + "," + e.Id + "," + e.Version + "'.");
        if (parameters != null)
        {
            foreach (var member in parameters.GetType().GetProperties())
            {
                var eventParams = e.DataNames;
                var i = eventParams.IndexOf(member.Name);
                Assert.AreNotEqual(-1, i, "ETW event does not have a data item named '" + member.Name + "'.");
                var value = member.GetValue(parameters);
                if (value != null)
                    Assert.AreEqual(value, e.DataValues[i], "ETW event data item mismatch for '" + member.Name + "'; expected '" + member.GetValue(parameters) + "' but observed '" + e.DataValues[i] + "'.");
            }
        }
        return @this;
#endif
        return @this;
    }

    public static void ExpectDone(this DebugEtw.EventCollection @this)
    {
#if NO
        var index = GetNextIndex(@this);
        if (index != @this.Count)
            Assert.Fail("Additional ETW events were raised.");
#endif
    }
}

[ExcludeFromCodeCoverage]
internal sealed class ExpectedEtwEvent
{
    public ExpectedEtwEvent(string sourceName, int id, int version)
    {
        SourceName = sourceName;
        Id = id;
        Version = version;
    }

    public string SourceName { get; private set; }
    public int Id { get; private set; }
    public int Version { get; private set; }
}
