﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics.CodeAnalysis;
using System.Linq;

[ExcludeFromCodeCoverage]
internal static class EtwRelations
{
    private static readonly string taskKey = "RelatedTasks_" + Guid.NewGuid().ToString("N");

    private static ISet<int> GetRelatedTasks(DebugEtw.Event e)
    {
        var dictionary = e.Bag as IDictionary<string, object>;
        if (dictionary.ContainsKey(taskKey))
            return (ISet<int>)dictionary[taskKey];

        var names = e.DataNames;
        var ret = new HashSet<int>();
        for (int i = 0; i != names.Count; ++i)
        {
            if (names[i].Contains("TaskID"))
            {
                int id = (int)e.DataValues[i];
                if (id != 0)
                    ret.Add(id);
            }
        }
        dictionary.Add(taskKey, ret);
        return ret;
    }

    public static bool HaveRelatedTasks(DebugEtw.Event a, DebugEtw.Event b)
    {
        var aTasks = GetRelatedTasks(a);
        var bTasks = GetRelatedTasks(b);
        return aTasks.Overlaps(bTasks);
    }
}